@@ -1,14 +1,17 @@
 from pathlib import Path
 from shutil import ExecError
 from typing import Any, Callable, Dict, List, Tuple, Union
+
+from tornado.locale import load_translations
 from experiment_server._participant_ordering import construct_participant_condition, ORDERING_BEHAVIOUR
-from experiment_server.utils import ExperimentServerConfigurationExcetion, merge_dicts
+from experiment_server.utils import ExperimentServerConfigurationExcetion, ExperimentServerExcetion, merge_dicts
 from loguru import logger
 from easydict import EasyDict as edict
 import json
+import toml
 
 
-TOP_LEVEL_RESERVED_KEYS = ["step_name", "config", "extends"]
+TOP_LEVEL_RESERVED_KEYS = ["name", "config", "extends"]
 SECTIONS = ["main_configuration", "init_configuration", "final_configuration", "template_values", "order", "settings"]
 ALLOWED_SETTINGS = ["randomize_within_groups", "randomize_groups"]
 
@@ -45,6 +48,46 @@ def process_config_file(f: Union[str, Path], participant_index: int) -> List[Dic
     if participant_index < 1:
         raise ExperimentServerConfigurationExcetion(f"Participant index needs to be greater than 0, got {participant_index}")
 
+    if Path(f).suffix == ".expconfig":
+        return _process_expconfig(f, participant_index)
+    elif Path(f).suffix == ".toml":
+        return _process_toml(f, participant_index)
+    else:
+        raise ExperimentServerExcetion("Invalid file type. Expected `.expconfig` or `.toml`")
+
+
+def _process_toml(f: Union[str, Path], participant_index:int) -> List[Dict[str, Any]]:
+    loaded_configuration = toml.load(f)
+
+    configurations = loaded_configuration.get("configuration", {})
+    variables = configurations.get("variables", {})
+    blocks = _replace_variables(loaded_configuration["blocks"], variables)
+
+    configurations_groups = configurations.get("groups", ORDERING_BEHAVIOUR.as_is)
+    configurations_within_groups = configurations.get("within_groups", ORDERING_BEHAVIOUR.as_is)
+
+    order = configurations.get("order", [list(range(len(blocks)))])
+
+    blocks = construct_participant_condition(blocks, participant_index, order=order,
+                                             groups=configurations_groups,
+                                             within_groups=configurations_within_groups)
+
+    init_blocks = _replace_variables(loaded_configuration.get("init_blocks", []), variables)
+    final_blocks = _replace_variables(loaded_configuration.get("final_blocks", []), variables)
+
+    blocks = init_blocks + blocks + final_blocks
+    # Using merge_dicts to ensure the values are references
+    blocks = resolve_extends([merge_dicts(b, {}) for b in blocks])
+
+    for c in blocks:
+        c["config"]["participant_index"] = participant_index
+        c["config"]["name"] = c["name"]
+    
+    logger.info("Configuration loaded: \n" + "\n".join([f"{idx}: {json.dumps(c, indent=2)}" for idx, c in enumerate(blocks)]))
+    return blocks
+
+
+def _process_expconfig(f: Union[str, Path], participant_index: int) -> List[Dict[str, Any]]:
     loaded_configurations = get_sections(f)
     if "template_values" in loaded_configurations:
         template_values = json.loads(loaded_configurations["template_values"])
@@ -57,8 +100,6 @@ def process_config_file(f: Union[str, Path], participant_index: int) -> List[Dic
     else:
         order = [list(range(len(loaded_configurations)))]
 
-    # TODO: expand repeat parameter
-
     if "settings" in loaded_configurations:
         settings = edict(json.loads(loaded_configurations["settings"]))
     else:
@@ -99,9 +140,12 @@ def process_config_file(f: Union[str, Path], participant_index: int) -> List[Dic
     config = init_configuration + main_configuration + final_configuration
     config = resolve_extends(config)
 
-    for c in config:
-        c["config"]["participant_index"] = participant_index
-        c["config"]["step_name"] = c["step_name"]
+    try:
+        for c in config:
+            c["config"]["participant_index"] = participant_index
+            c["config"]["name"] = c["name"]
+    except KeyError:
+        raise ExperimentServerConfigurationExcetion("blocks missing keys (config/name)")
     
     logger.info("Configuration loaded: \n" + "\n".join([f"{idx}: {json.dumps(c, indent=2)}" for idx, c in enumerate(config)]))
     return config
@@ -117,26 +161,26 @@ def _resolve_extends(c, configs, seen_configs):
 
     dict_a = c
     try:
-        dict_b = [_c for _c in configs if _c["step_name"] == c["extends"]][0]
+        dict_b = [_c for _c in configs if _c["name"] == c["extends"]][0]
     except IndexError:
-        raise ExperimentServerConfigurationExcetion("`{}` is not a valid name. It must be a `step_name`.".format(c["extends"]))
+        raise ExperimentServerConfigurationExcetion("`{}` is not a valid name. It must be a `name`.".format(c["extends"]))
 
-    dict_b, configs, seen_configs = _resolve_extends(dict_b, configs, seen_configs + [dict_b["step_name"]])
-    configs[dict_a["step_idx"]] = merge_dicts(dict_a, dict_b)
-    return configs[dict_a["step_idx"]], configs, seen_configs
+    dict_b, configs, seen_configs = _resolve_extends(dict_b, configs, seen_configs + [dict_b["name"]])
+    configs[dict_a["block_idx"]] = merge_dicts(dict_a, dict_b)
+    return configs[dict_a["block_idx"]], configs, seen_configs
 
 
 def resolve_extends(configs):
     # Adding idx to track the configs
     for idx, c in enumerate(configs):
-        c["step_idx"] = idx
+        c["block_idx"] = idx
 
     for c in configs:
-        configs[c["step_idx"]] = _resolve_extends(c, configs, [c["step_name"]])[0]
+        configs[c["block_idx"]] = _resolve_extends(c, configs, [c["name"]])[0]
 
     # Removing idx
     for c in configs:
-        del c["step_idx"]
+        del c["block_idx"]
 
     return configs
 
@@ -147,18 +191,46 @@ def _replace_template_values(string, template_values):
     return string
 
 
+def _replace_variables(config: Union[Dict[str, Any], List[Any]], variabels: Dict[str, Any]) -> Union[Dict[str, Any], List[Any]]:
+    if isinstance(config, dict):
+        resolved_config = {}
+        for k, v in config.items():
+            if isinstance(v, str) and v.startswith("$"):
+                try:
+                    resolved_config[k] = variabels[v[1:]]
+                except KeyError:
+                    raise ExperimentServerConfigurationExcetion(f"The variable `{v}` does not exsist in `configuration.variables`")
+            elif isinstance(v, dict):
+                resolved_config[k] = _replace_variables(v, variabels)
+            else:
+                resolved_config[k] = v
+    elif isinstance(config, list):
+        resolved_config = []
+        for v in config:
+            if isinstance(v, str) and v.startswith("$"):
+                try:
+                    resolved_config.append(variabels[v[1:]])
+                except KeyError:
+                    raise ExperimentServerConfigurationExcetion(f"The variable `{v}` does not exsist in `configuration.variables`")
+            elif isinstance(v, dict):
+                resolved_config.append(_replace_variables(v, variabels))
+            else:
+                resolved_config.append(v)
+    return resolved_config
+
+
 def verify_config(f: Union[str, Path], test_func:Callable[[List[Dict[str, Any]]], Tuple[bool, str]]=None) -> bool:
     import pandas as pd
     from tabulate import tabulate
     with logger.catch(reraise=False, message="Config verification failed"):
-        config_steps = {}
+        config_blocks = {}
         for participant_index in range(1,6):
             config = process_config_file(f, participant_index=participant_index)
-            config_steps[participant_index] = {f"trial_{idx + 1}": c["step_name"] for idx, c in enumerate(config)}
+            config_blocks[participant_index] = {f"trial_{idx + 1}": c["name"] for idx, c in enumerate(config)}
             if test_func is not None:
                 test_result, reason = test_func(config)
                 assert test_result, f"test_func failed for {participant_index} with reason, {reason}"
-        df = pd.DataFrame(config_steps)
+        df = pd.DataFrame(config_blocks)
         df.style.set_properties(**{'text-align': 'left'}).set_table_styles([ dict(selector='th', props=[('text-align', 'left')])])
         logger.info(f"Ordering for 5 participants: \n\n{tabulate(df, headers='keys', tablefmt='fancy_grid')}\n")
         logger.info(f"Config file verification successful for {f}")
